#!/usr/bin/env python
#$ -S /usr/bin/python
import sys
import math

#############################################################
# calc_rcell function
#############################################################
def calc_rcell(cell_string):
# obtaining each cell parameter
	cell=cell_string.split()
# Delete the last columns
	del cell[6:]
# deg->rad	
	d2r=math.pi/180.0
# Preparation
	a=float(cell[0])
	b=float(cell[1])
	c=float(cell[2])
	alph=float(cell[3])
	beta=float(cell[4])
	gamm=float(cell[5])
# Calculation simple coeficients
	sin_a=math.sin(alph*d2r)
	sin_b=math.sin(beta*d2r)
	sin_g=math.sin(gamm*d2r)
	cos_a=math.cos(alph*d2r)
	cos_b=math.cos(beta*d2r)
	cos_g=math.cos(gamm*d2r)
# Calculation of cos^2(alph)
	cos_a2=math.pow(cos_a,2.0)
	cos_b2=math.pow(cos_b,2.0)
	cos_g2=math.pow(cos_g,2.0)
	abc=a*b*c
# Calculation of cell volume
	v=abc*math.sqrt(1.0-cos_a2-cos_b2-cos_g2+2.0*cos_a*cos_b*cos_g)
# Reciprocal vectors
	cos_as=(cos_b*cos_g-cos_a)/(sin_b*sin_g)
	cos_bs=(cos_g*cos_a-cos_b)/(sin_g*sin_a)
	cos_gs=(cos_a*cos_b-cos_g)/(sin_a*sin_b)
	as=b*c*sin_a/v
	bs=c*a*sin_b/v
	cs=a*b*sin_g/v
# return reciprocal cell parameters
	rcell=[as,bs,cs,math.acos(cos_as),math.acos(cos_bs),math.acos(cos_gs)]
	return rcell

#############################################################
# calculating resolution
#############################################################
def calc_resol(h,k,l,rcell):
# preparation
	h2=h*h
	k2=k*k
	l2=l*l
# reciprocal cell parameters
	as=rcell[0]
	bs=rcell[1]
	cs=rcell[2]
	as2=as*as
	bs2=bs*bs
	cs2=cs*cs
	cos_as=math.cos(rcell[3])
	cos_bs=math.cos(rcell[4])
	cos_gs=math.cos(rcell[5])
# calculating d*
	ds2=h2*as2+k2*bs2+l2*cs2+2.0*(k*l*bs*cs*cos_as+l*h*cs*as*cos_bs+h*k*as*bs*cos_gs)
	ds=math.sqrt(ds2)
# storing resolution
	resol=1.0/ds
	return resol

#############################################################
# shell division
#############################################################
def which_shell(resmin,resmax,resol,nshell=10):
# calculating (dmin^3),(dmax^3),(d^3)
	d3min=math.pow(1/resmin,3.0)
	d3max=math.pow(1/resmax,3.0)
	d3cur=math.pow(1/resol,3.0)
# shell selection
	dwidth=(d3max-d3min)/nshell
	for i in range(0,10):
		tmp_min=d3min+i*dwidth
		tmp_max=d3min+(i+1)*dwidth
		if d3cur<tmp_max and d3cur>=tmp_min:
			return i
	return -1

#############################################################
# calculating median
#############################################################
def getMedian(resmin,resmax,nshell,idx):
# calculating (dmin^3),(dmax^3),(d^3)
	d3min=math.pow(1/resmin,3.0)
	d3max=math.pow(1/resmax,3.0)
# shell selection
	dwidth=(d3max-d3min)/nshell
	tmp_min=d3min+idx*dwidth
	tmp_max=d3min+(idx+1)*dwidth
	tmp_mean=(tmp_min+tmp_max)/2.0
	target=1.0/math.pow(tmp_mean,1/3.0)
	return target

#############################################################
# Main function
#############################################################
if len(sys.argv)!=4:
	print "Usage: readAnoSca SCAFILE MINRES MAXRES"
	sys.exit()
else:
	print "Processing %s"%sys.argv[1]

# File open
infile=open(sys.argv[1],'r')
# Minimum & Maximum resolution
minres=float(sys.argv[2])
maxres=float(sys.argv[3])

if minres < maxres:
	print "MINRES should be a larger value than MAXRES"
	sys.exit(0)

# reading all of lines
linelist=infile.readlines()

# initialization of parameters
nline=0	# line counter
nok=0 # number of reflections with anomalous
nshell=10

sumSN=[0]*nshell
cntSN=[0]*nshell

# acquiring cell parameters at the 3rd line
cell_string=linelist[2]
rcell=calc_rcell(cell_string)

# treating each line
for line in linelist:
# counting a number of line
	nline+=1
# initialization
	strings=line.split()
# counting a number of columns
	ncolumn=len(strings)
# data processing
# ncolumn=7: anomalous signals can be obtained
# ncolumn=5: anomalous signals cannot be obtained
	if ncolumn==7 and nline>4:
	# acquiring HKL indices
		h=int(strings[0])
		k=int(strings[1])
		l=int(strings[2])
	# calculating resolution
		resol=calc_resol(h,k,l,rcell)
		if resol < minres and resol > maxres:
		# choosing which shell
			idx=which_shell(minres,maxres,resol,nshell)
			if idx==-1:
				print "Critical error!\n"
				continue
		else: 
			continue
	# for I(+)s
		i_plus=float(strings[3])
		sig_plus=float(strings[4])
	# case when I(+) was not observed -> skip this reflection
		if i_plus==0 and sig_plus==-1.0:
			continue
	# for I(-)s
		i_minus=float(strings[5])
		sig_minus=float(strings[6])
	# difference between I(+) and I(-)
		nok+=1
		bunshi=math.fabs(i_plus-i_minus)
		bunbo=(sig_plus+sig_minus)/2.0
		snvalue=bunshi/bunbo
		if snvalue>0.0:
			sumSN[idx]+=snvalue
			cntSN[idx]+=1

# Final calculation of average
for i in range(0,nshell):
	median=getMedian(minres,maxres,nshell,i)
	dstar2=1.0/median/median
	print "%5.2f %5.3f %5d %8.2f" % (median,dstar2,cntSN[i],sumSN[i]/cntSN[i])

